//region Imports
const path = require('path')
const _ = require('lodash')
const chalk = require('chalk')
const config = require('./config')
const fs = require('fs')
//endregion

function pp(...args) {
  const prettyjson = require('prettyjson')

  console.log(prettyjson.render(...args))
}

const tasksDir = './tasks'

function requireDir(fullDir) {
  return require('fs').readdirSync(fullDir).map((entry) => {
    try {
      return require(path.join(fullDir, entry))
    } catch (e) {
      if (e.name !== 'MODULE_NOT_FOUND') {
        throw e
      }
    }
  })
}

function d(description, fn) {
  fn.description = description
  return fn
}

const main = function(gulp) {
  const FwdRef = require('undertaker-forward-reference')

  // TODO(vjpr): Extract common `src/dist` paths.
  const opts = config()

  // This allows forward references to tasks.
  const fwdReferenceRegistry = new FwdRef
  const registry = fwdReferenceRegistry
  gulp.registry(fwdReferenceRegistry)

  require('gulp-help')(gulp, {
    hideDefaultTask: true,
    hideEmpty: true, // Doesn't exist yet...https://github.com/chmontgomery/gulp-help/issues/36
  })

  // Require all modules in `./tasks` dir, and pass `gulp` and `opts` to each module.

  const fullDir = path.join(__dirname, tasksDir)
  const modules = requireDir(fullDir)
  console.log(modules)
  _(modules).each((module) => {
    module(gulp, opts)
  }).run()

  gulp.task('default', gulp.series('help'))
  gulp.task('compile', d(`Compile ${chalk.bold(opts.srcDir)} to ${chalk.bold(opts.destDir)}`, gulp.series('lib:compile')))
  gulp.task('test', d('Compile and run tests on transpiled tests', gulp.series('mocha:lib')))
  gulp.task('test-src', d(`Run tests using glob: ${chalk.bold(opts.testsGlob)}`, gulp.series('mocha:src')))
  gulp.task('watch', d('Compile on file change', gulp.series('lib:watch')))
  gulp.task('test-watch', d('Compile and run tests on file change', gulp.series('test:watch')))
  gulp.task('build', d('Clean and compile', gulp.series('lib:clean', 'lib:compile')))
  gulp.task('show-config', d('Show resolved config settings', (done) => {
    console.log(`\n${chalk.underline('Resolved config')}\n`)
    pp(opts)
    console.log('')
    const p = config.getConfigPath()
    if (fs.existsSync(p)) {
      console.log('Using config from:', chalk.bold(p))
      console.log('All possible config options can be found by looking in:', chalk.bold(tasksDir))
    } else {
      console.log('To modify config created file:', p)
    }
    console.log('')
    done()
  }))

  // Run all tasks ending with `:bootstrap`.
  const bootstrapTasks = _(registry._tasks).pick((v, k) => _.endsWith(k, ':bootstrap')).run()
  gulp.task('bootstrap', gulp.series(_.values(bootstrapTasks)))
  gulp.task('prepublish', gulp.series('npm:prepublish'))

}

module.exports = main
