//region Imports
const path = require('path')
const _ = require('lodash')
//endregion

// Babel logging.
function logFileHelpers() {
  const through = require('through2')
  const gutil = require('gulp-util')

  return through.obj(function(file, enc, cb) {
    var src = file.history[0]
    var dest = file.history[1]
    const relPath = (f) => { return path.relative('.', f) }
    gutil.log('Compiling (Babel):', gutil.colors.red(relPath(src) + ' -> ' + relPath(dest)))
    //gutil.log(file.babel.usedHelpers)
    cb(null, file)
  })
}

function onError() {
  const notify = require('gulp-notify')
  return notify.onError('<%= error.message %>')
  //return notify.onError((e) => {
  //  console.log(e)
  //  const filename = path.relative('.', e.fileName)
  //  const moduleName = path.basename(process.cwd())
  //  return `${moduleName} - ${filename}\n ${e.message}`
  //  // TODO(vjpr): Not working.
  //  //return {
  //  //  title: 'babel compile error',
  //  //  message: `[${moduleName}] ${filename}`,
  //  //}
  //})
}

export default (gulp, opts) => {

  ////////
  // This is shared.
  const asyncPlugin =
    ['transform-async-to-module-method', {
      module: 'bluebird',
      method: 'coroutine',
    }]
  const addModuleExports = require('babel-plugin-add-module-exports')
  const transformRuntime = require('babel-plugin-transform-runtime')
  const externalHelpers = require('babel-plugin-external-helpers-2')
  const babelDefaults = {
    plugins: [externalHelpers, asyncPlugin, transformRuntime, addModuleExports],
    presets: ['es2015', 'stage-0']
  }
  ////////

  _.defaults(opts, {
    srcGlob: '',
    shouldCompileTests: false,
    destDir: '',
    testsGlob: '',
    babel: babelDefaults,
  })

  gulp.task('lib:compile', () => {
    const gulpif = require('gulp-if')
    const changed = require('gulp-changed')
    const ignore = require('gulp-ignore')
    const babel = require('gulp-babel')
    const sourcemaps = require('gulp-sourcemaps')
    const cwd = require('cwd')

    return gulp.src(opts.srcGlob)
      .pipe(gulpif(gulp.isWatching, changed(opts.destDir)))
      .pipe(gulpif(!opts.shouldCompileTests, ignore.exclude(opts.testsGlob)))
      .pipe(sourcemaps.init())
      .pipe(babel(opts.babel))
      .on('error', onError())
      // Source maps work with IntelliJ debugger using Node.js 0.12.7.
      .pipe(sourcemaps.write('.', {
        includeContent: false,
        sourceRoot: file => {
          return path.join(path.relative(file.path, cwd()), opts.srcDir)
        }
      }))
      .pipe(gulp.dest(opts.destDir))
      .pipe(logFileHelpers())
  })

  gulp.task('lib:clean', () => {
    const del = require('del')
    const gutil = require('gulp-util')

    gutil.log('Cleaning:', gutil.colors.red(opts.destDir))
    return del(opts.destGlob)
  })

  gulp.task('lib:watch', gulp.series('lib:clean', 'lib:compile', function watcher() {
    const gutil = require('gulp-util')

    // Used by `compile` task to check whether we should only compile changed files.
    gulp.isWatching = true

    gulp.watch(opts.srcGlob, gulp.series('lib:compile')).on('change', (evt) => {
      gutil.log('---')
      gutil.log('File changed:', gutil.colors.red(evt))
    })

  }))

}
